#!/bin/sh -
# Suck in the configuration variables.
if [ -z "${source_rc_confs_defined}" ]; then
	if [ -r /etc/defaults/rc.conf ]; then
		. /etc/defaults/rc.conf
		source_rc_confs
	elif [ -r /etc/rc.conf ]; then
		. /etc/rc.conf
	fi
fi

if [ -n "${1}" ]; then
	firewall_type="${1}"
fi

############
# Set quiet mode if requested
#
case ${firewall_quiet} in
[Yy][Ee][Ss])
	fwcmd="/sbin/ipfw -q"
	;;
*)
	fwcmd="/sbin/ipfw"
	;;
esac

############
# Flush out the list before we begin.
#
${fwcmd} -f flush

case ${firewall_type} in
[Mm][Yy][Ss][Ii][Mm][Pp][Ll][Ee])
	# ==============================================================
	# This is my ruleset for ipfw(8). 20 March 2005
	# ==============================================================

	# Set these to your outside interface network and netmask and ip
	oif="ng0"
	oip="my.ip.add.ress"

	# Set these to your inside interface network and netmask and ip
	iif="rl0"
	iip="192.168.82.253"
	inet="192.168.82.0/24"
	
	# Setup Loopback
	${fwcmd} add 110 set 0 pass all from any to any via lo0
	${fwcmd} add 120 set 0 deny all from any to 127.0.0.0/8
	${fwcmd} add 130 set 0 deny ip from 127.0.0.0/8 to any

	${fwcmd} add 200 set 1 deny log ip from any to any
	

	${fwcmd} add 5000 set 0 skipto 10000 all from any to any in recv ${oif}
	${fwcmd} add 5010 set 0 skipto 20000 all from any to any out xmit ${oif}
	${fwcmd} add 5020 set 0 skipto 30000 all from any to any in recv ${iif}
	${fwcmd} add 5030 set 0 skipto 40000 all from any to any out xmit ${iif} 
	
	${fwcmd} add 5080 set 0 allow all from any to any via rl1

	${fwcmd} add 5100 set 0 deny log all from any to any
	#########################################################
	# Process packets in via ${oif} #########################
	#########################################################

	${fwcmd} add 10000 set 0 count all from any to any in recv ${oif}

	# Spoofing
	${fwcmd} add 10100 set 0 deny all from any to 192.168.0.0/16 in via ${oif}
	${fwcmd} add 10110 set 0 deny all from 192.168.0.0/16 to any in via ${oif}

	# Reset ident incoming connections
	${fwcmd} add 10200 set 0 deny log tcp from any to me 113 in recv ${oif} setup

	# Deny & log suspicious packets (like nmap scans)
	${fwcmd} add 10210 set 0 deny log tcp from any to any in tcpflags syn,fin

	# Here comes NAT ************************
	${fwcmd} add 15000 set 0 divert natd all from any to me in via ${oif}
	#****************************************
	
	# Here come the dynamic rules
	# NOTE: check-state here will also process rules created by
	# keep-state from "in recv ${iif}" !
	${fwcmd} add 16000 set 0 check-state
	${fwcmd} add 16010 set 0 deny tcp from any to me established

	# NOTE:  limit src-addr dst-addr N  is for DoS protection
	# Allow access to our FTP (20,21), SSH (22), SMTP (25), DNS (53), POP3-ssl (995)
	${fwcmd} set disable 11
	${fwcmd} add 16100 set 11 pass tcp from any to me 20,21 in recv ${oif} setup limit src-addr 4
	${fwcmd} add 16200 set 12 pass tcp from any to me 22 in recv ${oif} setup limit src-addr 4
	${fwcmd} add 16300 set 13 pass tcp from any to me 25 in recv ${oif} setup keep-state
	${fwcmd} add 16400 set 14 pass tcp from any to me 53 in recv ${oif} setup keep-state
	${fwcmd} add 16500 set 15 pass tcp from any to me 995 in recv ${oif} setup limit src-addr 4
	${fwcmd} add 16600 set 16 pass udp from any to me 53 in recv ${oif} keep-state

	# Allow some icmp
	# echo reply (0), destination unreachable (3), source quench (4), echo request 
	# (8), time-to-live exceeded (11), IP header bad (12)
	#${fwcmd} add 17100 set 0 pass icmp from any to any icmptype 0,3,4,8,11,12
	#${fwcmd} add 17101 set 0 deny icmp from any to any
	${fwcmd} add 17100 set 0 pass icmp from any to any

	# Allow IP fragments to pass through
	${fwcmd} add 17200 set 0 pass all from any to any frag

	# And enough
	${fwcmd} add 19000 set 0 skipto 65000 all from any to any in recv ${oif}

	#########################################################
	# Process packets out via ${oif} ########################
	#########################################################

	${fwcmd} add 20000 set 0 count all from any to any out xmit ${oif}

	# Spoofing
	${fwcmd} add 20100 set 0 deny all from any to 192.168.0.0/16 out xmit ${oif}


	# Here comes NAT********************
	${fwcmd} add 25000 set 0 divert natd all from ${inet} to any out xmit ${oif}
	# **********************************
	
	# NOTE: check-state here does nothing.
	# keep-state is only for ME. Local net will not use created rules.
	${fwcmd} add 26000 set 0 check-state
	${fwcmd} add 26100 set 0 allow tcp from me to any out xmit ${oif} keep-state
	${fwcmd} add 26200 set 0 allow udp from me to any out xmit ${oif} keep-state

	# Allow some icmp
	# echo reply (0), destination unreachable (3), source quench (4), echo request 
	# (8), time-to-live exceeded (11), IP header bad (12)
	#${fwcmd} add 26300 set 0 pass icmp from any to any icmptype 0,3,4,8,11,12
	#${fwcmd} add 26301 set 0 deny icmp from any to any
	${fwcmd} add 26300 set 0 pass icmp from any to any

	# Allow IP fragments to pass through
	${fwcmd} add 26400 set 0 pass all from any to any frag
	
	# Enough
	${fwcmd} add 29000 set 0 skipto 65000 all from any to any out xmit ${oif}

	#########################################################
	# Process packets in via ${iif} #########################
	#########################################################


	${fwcmd} add 30000 set 0 count all from any to any in recv ${iif}

	${fwcmd} add 35000 set 0 check-state
	${fwcmd} add 35100 set 0 deny tcp from any to me established in recv ${iif}
	${fwcmd} add 35200 set 0 allow tcp from ${inet} to any in recv ${iif} setup keep-state
	${fwcmd} add 35300 set 0 allow udp from ${inet} to any in recv ${iif} keep-state

	# Allow some icmp
	# echo reply (0), destination unreachable (3), source quench (4), echo request 
	# (8), time-to-live exceeded (11), IP header bad (12)
	#${fwcmd} add 36000 set 0 pass icmp from any to any icmptype 0,3,4,8,11,12
	#${fwcmd} add 36001 set 0 deny icmp from any to any
	${fwcmd} add 36000 set 0 pass icmp from any to any

	# Allow IP fragments to pass through
	${fwcmd} add 36100 set 0 pass all from any to any frag

	# Enough
	${fwcmd} add 39000 set 0 skipto 65000 all from any to any in recv ${iif}

	#########################################################
	# Process packets out via ${iif} ########################
	#########################################################

	${fwcmd} add 40000 set 0 count all from any to any out xmit ${iif}

	# NOTE: only packets from ME have no dynamicly formed rules now

	${fwcmd} add 45000 set 0 check-state
	${fwcmd} add 45100 set 0 allow tcp from me to any keep-state
	${fwcmd} add 45200 set 0 allow udp from me to any keep-state
	${fwcmd} add 45250 set 0 allow all from any to ${emule}

	${fwcmd} add 45300 set 0 allow all from 192.168.0.0/16 to ${inet} keep-state
	

	# Client-bank. No sense. Make it clear.
	#${fwcmd} add 47300 set 30 allow log tcp from any 20 to ${inet}
	# And this one is on passive mode? Packets come from ports 30000+
	#${fwcmd} add 47310 set 30 allow log tcp from 80.249.225.110 to ${inet}

	# Allow some icmp
	# echo reply (0), destination unreachable (3), source quench (4), echo request 
	# (8), time-to-live exceeded (11), IP header bad (12)
	#${fwcmd} add 48000 set 0 pass icmp from any to any icmptype 0,3,4,8,11,12
	#${fwcmd} add 48001 set 0 deny icmp from any to any
	${fwcmd} add 48000 set 0 pass icmp from any to any

	# Allow IP fragments to pass through
	${fwcmd} add 48100 set 0 pass all from any to any frag

	########################################################
	# That's all, folks deny and log everything ############
	########################################################
	# Reject & log all other udp connections
	${fwcmd} add 65000 set 5 deny log udp from any to any

	# Reject & log everything else
	${fwcmd} add 65100 set 5 deny log all from any to any

	# Disable rule 200, open firewall for work
	${fwcmd} set disable 1
	#======================================================================
	;;
	
*)
	if [ -r "${firewall_type}" ]; then
		${fwcmd} ${firewall_flags} ${firewall_type}
	fi
	;;
esac